@@ -282,7 +282,7 @@ xsltParseStylesheetAttributeSet(xsltStylesheetPtr style, xmlNodePtr cur) {
     if ((cur == NULL) || (style == NULL))
 	return;
 
-    prop = xsltGetNsProp(cur, (const xmlChar *)"name", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"name", NULL);
     if (prop == NULL) {
 	xsltGenericError(xsltGenericErrorContext,
 	     "xsl:attribute-set : name is missing\n");
@@ -333,8 +333,8 @@ xsltParseStylesheetAttributeSet(xsltStylesheetPtr style, xmlNodePtr cur) {
      */
     /* TODO check recursion */
 
-    attributes = xsltGetNsProp(cur, (const xmlChar *)"use-attribute-sets",
-	                      XSLT_NAMESPACE);
+    attributes = xmlGetNsProp(cur, (const xmlChar *)"use-attribute-sets",
+	NULL);
     if (attributes == NULL) {
 	goto done;
     }
@@ -565,9 +565,15 @@ xsltResolveStylesheetAttributeSet(xsltStylesheetPtr style) {
  */
 static void
 xsltAttributeInternal(xsltTransformContextPtr ctxt, xmlNodePtr node,
-                      xmlNodePtr inst, xsltStylePreCompPtr comp,
-                      int fromset)
-{
+                      xmlNodePtr inst,
+		      xsltStylePreCompPtr castedComp,
+                      int fromset) {
+#ifdef XSLT_REFACTORED
+    xsltStyleItemAttributePtr comp =
+	(xsltStyleItemAttributePtr) castedComp;
+#else
+    xsltStylePreCompPtr comp = castedComp;
+#endif
     xmlChar *prop = NULL;
     xmlChar *namespace;
     const xmlChar *name = NULL;
@@ -578,16 +584,15 @@ xsltAttributeInternal(xsltTransformContextPtr ctxt, xmlNodePtr node,
     const xmlChar *URL = NULL;
 
 
-    if (ctxt->insert == NULL)
-        return;
-    if (comp == NULL) {
+    if ((comp == NULL) || (ctxt == NULL)) {
         xsltTransformError(ctxt, NULL, inst,
                          "xsl:attribute : compilation failed\n");
         return;
     }
+    if (ctxt->insert == NULL)
+        return;
 
-    if ((ctxt == NULL) || (node == NULL) || (inst == NULL)
-        || (comp == NULL))
+    if ((node == NULL) || (inst == NULL) || (comp == NULL))
         return;
     if (!comp->has_name) {
         return;
