@@ -438,7 +438,7 @@ xsltAttrTemplateProcess(xsltTransformContextPtr ctxt, xmlNodePtr target,
     const xmlChar *value;
     xmlNsPtr ns;
     xmlAttrPtr ret;
-    if ((ctxt == NULL) || (cur == NULL))
+    if ((ctxt == NULL) || (cur == NULL) || (target == NULL))
 	return(NULL);
     
     if (cur->type != XML_ATTRIBUTE_NODE)
@@ -451,7 +451,8 @@ xsltAttrTemplateProcess(xsltTransformContextPtr ctxt, xmlNodePtr target,
         return(NULL);
     }
     value = cur->children->content;
-    if (value == NULL) value = BAD_CAST "";
+    if (value == NULL)
+        value = xmlDictLookup(ctxt->dict, BAD_CAST "", 0);
     if ((cur->ns != NULL) &&
 	(xmlStrEqual(cur->ns->href, XSLT_NAMESPACE))) {
 	if (xmlStrEqual(cur->name, (const xmlChar *)"use-attribute-sets")) {
@@ -502,6 +503,10 @@ xsltAttrTemplateProcess(xsltTransformContextPtr ctxt, xmlNodePtr target,
 		} else {
 		    text->content = val;
 		}
+	    } else if ((ctxt->internalized) && (target != NULL) &&
+	               (target->doc != NULL) &&
+		       (target->doc->dict == ctxt->dict)) {
+		text->content = (xmlChar *) value;
 	    } else {
 		text->content = xmlStrdup(value);
 	    }
