@@ -22,6 +22,7 @@
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
 #include <libxml/parserInternals.h>
+#include <libxml/dict.h>
 #include "xslt.h"
 #include "xsltInternals.h"
 #include "xsltutils.h"
@@ -250,6 +251,8 @@ xsltFreeStackElemList(xsltStackElemPtr elem) {
  *
  * Locate an element in the stack based on its name.
  */
+static int stack_addr = 0;
+static int stack_cmp = 0;
 static xsltStackElemPtr
 xsltStackLookup(xsltTransformContextPtr ctxt, const xmlChar *name,
 	        const xmlChar *nameURI) {
@@ -271,11 +274,13 @@ xsltStackLookup(xsltTransformContextPtr ctxt, const xmlChar *name,
 	    if (cur->name == name) {
 		if (nameURI == NULL) {
 		    if (cur->nameURI == NULL) {
+		        stack_addr++;
 			return(cur);
 		    }
 		} else {
 		    if ((cur->nameURI != NULL) &&
 			(cur->nameURI == nameURI)) {
+		        stack_addr++;
 			return(cur);
 		    }
 		}
@@ -285,47 +290,28 @@ xsltStackLookup(xsltTransformContextPtr ctxt, const xmlChar *name,
 	}
     }
 
-#if 0
-    if ((xmlDictOwns(ctxt->dict, name) <= 0) ||
-        ((nameURI != NULL) && (xmlDictOwns(ctxt->dict, nameURI) <= 0))) {
-	/*
-	 * Redo the lookup with string compares
-	 */
-	for (i = ctxt->varsNr; i > ctxt->varsBase; i--) {
-	    cur = ctxt->varsTab[i-1];
-	    while (cur != NULL) {
-		if (xmlStrEqual(cur->name, name)) {
-		    if (nameURI == NULL) {
-			if (cur->nameURI == NULL) {
-			    return(cur);
-			}
-		    } else {
-			if ((cur->nameURI != NULL) &&
-			    (xmlStrEqual(cur->nameURI, nameURI))) {
-			    return(cur);
-			}
-		    }
-
-		}
-		cur = cur->next;
-	    }
-	}
-    }
-#else
     /*
-     * Redo the lookup with string compares
+     * Redo the lookup with interned string compares
+     * to avoid string compares.
      */
+    name = xmlDictLookup(ctxt->dict, name, -1);
+    if (nameURI != NULL)
+        nameURI = xmlDictLookup(ctxt->dict, nameURI, -1);
+    else
+        nameURI = NULL;
     for (i = ctxt->varsNr; i > ctxt->varsBase; i--) {
 	cur = ctxt->varsTab[i-1];
 	while (cur != NULL) {
-	    if (xmlStrEqual(cur->name, name)) {
+	    if (cur->name == name) {
 		if (nameURI == NULL) {
 		    if (cur->nameURI == NULL) {
+		        stack_cmp++;
 			return(cur);
 		    }
 		} else {
 		    if ((cur->nameURI != NULL) &&
-			(xmlStrEqual(cur->nameURI, nameURI))) {
+			(cur->nameURI == nameURI)) {
+		        stack_cmp++;
 			return(cur);
 		    }
 		}
@@ -334,7 +320,7 @@ xsltStackLookup(xsltTransformContextPtr ctxt, const xmlChar *name,
 	    cur = cur->next;
 	}
     }
-#endif
+
     return(NULL);
 }
 
@@ -438,7 +424,14 @@ xsltAddStackElemList(xsltTransformContextPtr ctxt, xsltStackElemPtr elems) {
  */
 static xmlXPathObjectPtr
 xsltEvalVariable(xsltTransformContextPtr ctxt, xsltStackElemPtr elem,
-	         xsltStylePreCompPtr precomp) {
+	         xsltStylePreCompPtr castedComp)
+{
+#ifdef XSLT_REFACTORED
+    xsltStyleItemVariablePtr precomp =
+	(xsltStyleItemVariablePtr) castedComp;
+#else
+    xsltStylePreCompPtr precomp = castedComp;
+#endif   
     xmlXPathObjectPtr result = NULL;
     int oldProximityPosition, oldContextSize;
     xmlNodePtr oldInst, oldNode;
@@ -473,8 +466,18 @@ xsltEvalVariable(xsltTransformContextPtr ctxt, xsltStackElemPtr elem,
 	oldNamespaces = ctxt->xpathCtxt->namespaces;
 	if (precomp != NULL) {
 	    ctxt->inst = precomp->inst;
+#ifdef XSLT_REFACTORED
+	    if (precomp->inScopeNS != NULL) {
+		ctxt->xpathCtxt->namespaces = precomp->inScopeNS->list;
+		ctxt->xpathCtxt->nsNr = precomp->inScopeNS->number;
+	    } else {
+		ctxt->xpathCtxt->namespaces = NULL;
+		ctxt->xpathCtxt->nsNr = 0;
+	    }
+#else
 	    ctxt->xpathCtxt->namespaces = precomp->nsList;
 	    ctxt->xpathCtxt->nsNr = precomp->nsNr;
+#endif
 	} else {
 	    ctxt->inst = NULL;
 	    ctxt->xpathCtxt->namespaces = NULL;
@@ -491,8 +494,12 @@ xsltEvalVariable(xsltTransformContextPtr ctxt, xsltStackElemPtr elem,
 	if ((precomp == NULL) || (precomp->comp == NULL))
 	    xmlXPathFreeCompExpr(comp);
 	if (result == NULL) {
-	    xsltTransformError(ctxt, NULL, precomp->inst,
-		"Evaluating variable %s failed\n", elem->name);
+	    if (precomp == NULL)
+		xsltTransformError(ctxt, NULL, NULL,
+		    "Evaluating variable %s failed\n", elem->name);
+	    else
+		xsltTransformError(ctxt, NULL, precomp->inst,
+		    "Evaluating variable %s failed\n", elem->name);
 	    ctxt->state = XSLT_STATE_STOPPED;
 #ifdef WITH_XSLT_DEBUG_VARIABLE
 #ifdef LIBXML_DEBUG_ENABLED
@@ -559,10 +566,16 @@ xsltEvalVariable(xsltTransformContextPtr ctxt, xsltStackElemPtr elem,
  * Returns the XPath Object value or NULL in case of error
  */
 static xmlXPathObjectPtr
-xsltEvalGlobalVariable(xsltStackElemPtr elem, xsltTransformContextPtr ctxt) {
+xsltEvalGlobalVariable(xsltStackElemPtr elem, xsltTransformContextPtr ctxt)
+{
     xmlXPathObjectPtr result = NULL;
+#ifdef XSLT_REFACTORED
+    xsltStyleBasicItemVariablePtr precomp;
+#else
     xsltStylePreCompPtr precomp;
+#endif    
     int oldProximityPosition, oldContextSize;
+    xmlDocPtr oldDoc;
     xmlNodePtr oldInst;
     int oldNsNr;
     xmlNsPtr *oldNamespaces;
@@ -588,7 +601,11 @@ xsltEvalGlobalVariable(xsltStackElemPtr elem, xsltTransformContextPtr ctxt) {
     name = elem->name;
     elem->name = BAD_CAST "  being computed ... ";
 
+#ifdef XSLT_REFACTORED
+    precomp = (xsltStyleBasicItemVariablePtr) elem->comp;
+#else
     precomp = elem->comp;
+#endif
     if (elem->select != NULL) {
 	xmlXPathCompExprPtr comp = NULL;
 
@@ -601,22 +618,37 @@ xsltEvalGlobalVariable(xsltStackElemPtr elem, xsltTransformContextPtr ctxt) {
 	    elem->name = name;
 	    return(NULL);
 	}
+	oldDoc = ctxt->xpathCtxt->doc;
 	oldProximityPosition = ctxt->xpathCtxt->proximityPosition;
 	oldContextSize = ctxt->xpathCtxt->contextSize;
 	oldInst = ctxt->inst;
 	oldNsNr = ctxt->xpathCtxt->nsNr;
 	oldNamespaces = ctxt->xpathCtxt->namespaces;
+	
 	if (precomp != NULL) {
 	    ctxt->inst = precomp->inst;
+#ifdef XSLT_REFACTORED
+	    if (precomp->inScopeNS != NULL) {
+		ctxt->xpathCtxt->namespaces = precomp->inScopeNS->list;
+		ctxt->xpathCtxt->nsNr = precomp->inScopeNS->number;
+	    } else {
+		ctxt->xpathCtxt->namespaces = NULL;
+		ctxt->xpathCtxt->nsNr = 0;
+	    }
+#else
 	    ctxt->xpathCtxt->namespaces = precomp->nsList;
 	    ctxt->xpathCtxt->nsNr = precomp->nsNr;
+#endif
 	} else {
 	    ctxt->inst = NULL;
 	    ctxt->xpathCtxt->namespaces = NULL;
 	    ctxt->xpathCtxt->nsNr = 0;
 	}
-	ctxt->xpathCtxt->node = (xmlNodePtr) ctxt->node;
+	ctxt->xpathCtxt->doc = ctxt->tmpDoc;
+	ctxt->xpathCtxt->node = (xmlNodePtr) ctxt->tmpDoc;
 	result = xmlXPathCompiledEval(comp, ctxt->xpathCtxt);
+
+	ctxt->xpathCtxt->doc = oldDoc;
 	ctxt->xpathCtxt->contextSize = oldContextSize;
 	ctxt->xpathCtxt->proximityPosition = oldProximityPosition;
 	ctxt->inst = oldInst;
@@ -625,8 +657,12 @@ xsltEvalGlobalVariable(xsltStackElemPtr elem, xsltTransformContextPtr ctxt) {
 	if ((precomp == NULL) || (precomp->comp == NULL))
 	    xmlXPathFreeCompExpr(comp);
 	if (result == NULL) {
-	    xsltTransformError(ctxt, NULL, precomp->inst,
-		"Evaluating global variable %s failed\n", elem->name);
+	    if (precomp == NULL)
+		xsltTransformError(ctxt, NULL, NULL,
+		    "Evaluating global variable %s failed\n", elem->name);
+	    else
+		xsltTransformError(ctxt, NULL, precomp->inst,
+		    "Evaluating global variable %s failed\n", elem->name);
 	    ctxt->state = XSLT_STATE_STOPPED;
 #ifdef WITH_XSLT_DEBUG_VARIABLE
 #ifdef LIBXML_DEBUG_ENABLED
@@ -714,6 +750,7 @@ xsltEvalGlobalVariables(xsltTransformContextPtr ctxt) {
 	"Registering global variables\n"));
 #endif
 
+    ctxt->tmpDoc = ctxt->document->doc;
     ctxt->node = (xmlNodePtr) ctxt->document->doc;
     ctxt->xpathCtxt->contextSize = 1;
     ctxt->xpathCtxt->proximityPosition = 1;
@@ -947,6 +984,8 @@ xsltProcessUserParamInternal(xsltTransformContextPtr ctxt,
 	xsltTransformError(ctxt, style, NULL,
 	    "Global parameter %s already defined\n", name);
     }
+    if (ctxt->globalVars == NULL)
+	ctxt->globalVars = xmlHashCreate(20);
 
     /*
      * do not overwrite variables with parameters from the command line
@@ -1026,11 +1065,8 @@ xsltProcessUserParamInternal(xsltTransformContextPtr ctxt,
     elem = xsltNewStackElem();
     if (elem != NULL) {
 	elem->name = name;
-	if (value != NULL)
-	    elem->select = xmlDictLookup(ctxt->dict, value, -1);
-	else
-	    elem->select = NULL;
-	if (href)
+	elem->select = xmlDictLookup(ctxt->dict, value, -1);
+	if (href != NULL)
 	    elem->nameURI = xmlDictLookup(ctxt->dict, href, -1);
 	elem->tree = NULL;
 	elem->computed = 1;
@@ -1175,8 +1211,16 @@ xsltQuoteOneUserParam(xsltTransformContextPtr ctxt,
  * Returns the xsltStackElemPtr or NULL in case of error
  */
 static xsltStackElemPtr
-xsltBuildVariable(xsltTransformContextPtr ctxt, xsltStylePreCompPtr comp,
-		  xmlNodePtr tree) {
+xsltBuildVariable(xsltTransformContextPtr ctxt,
+		  xsltStylePreCompPtr castedComp,
+		  xmlNodePtr tree)
+{
+#ifdef XSLT_REFACTORED
+    xsltStyleBasicItemVariablePtr comp =
+	(xsltStyleBasicItemVariablePtr) castedComp;
+#else
+    xsltStylePreCompPtr comp = castedComp;
+#endif 
     xsltStackElemPtr elem;
 
 #ifdef WITH_XSLT_DEBUG_VARIABLE
@@ -1191,7 +1235,7 @@ xsltBuildVariable(xsltTransformContextPtr ctxt, xsltStylePreCompPtr comp,
     elem = xsltNewStackElem();
     if (elem == NULL)
 	return(NULL);
-    elem->comp = comp;
+    elem->comp = (xsltStylePreCompPtr) comp;
     elem->name = comp->name;
     if (comp->select != NULL)
 	elem->select = comp->select;
@@ -1201,7 +1245,8 @@ xsltBuildVariable(xsltTransformContextPtr ctxt, xsltStylePreCompPtr comp,
 	elem->nameURI = comp->ns;
     elem->tree = tree;
     if (elem->computed == 0) {
-	elem->value = xsltEvalVariable(ctxt, elem, comp);
+	elem->value = xsltEvalVariable(ctxt, elem,
+	    (xsltStylePreCompPtr) comp);
 	if (elem->value != NULL)
 	    elem->computed = 1;
     }
@@ -1215,13 +1260,22 @@ xsltBuildVariable(xsltTransformContextPtr ctxt, xsltStylePreCompPtr comp,
  * @tree:  the tree if select is NULL
  * @param:  this is a parameter actually
  *
- * Computes and register a new variable value. 
+ * Computes and register a new variable value.
+ * TODO: Is this intended for xsl:param as well?
  *
  * Returns 0 in case of success, -1 in case of error
  */
 static int
-xsltRegisterVariable(xsltTransformContextPtr ctxt, xsltStylePreCompPtr comp,
-		     xmlNodePtr tree, int param) {
+xsltRegisterVariable(xsltTransformContextPtr ctxt,
+		     xsltStylePreCompPtr castedComp,
+		     xmlNodePtr tree, int param)
+{
+#ifdef XSLT_REFACTORED
+    xsltStyleBasicItemVariablePtr comp =
+	(xsltStyleBasicItemVariablePtr) castedComp;
+#else
+    xsltStylePreCompPtr comp = castedComp;
+#endif
     xsltStackElemPtr elem;
     int present;
 
@@ -1244,7 +1298,7 @@ xsltRegisterVariable(xsltTransformContextPtr ctxt, xsltStylePreCompPtr comp,
 #endif
 	return(0);
     }
-    elem = xsltBuildVariable(ctxt, comp, tree);
+    elem = xsltBuildVariable(ctxt, (xsltStylePreCompPtr) comp, tree);
     xsltAddStackElem(ctxt, elem);
     return(0);
 }
@@ -1344,14 +1398,23 @@ xsltVariableLookup(xsltTransformContextPtr ctxt, const xmlChar *name,
  */
 
 xsltStackElemPtr
-xsltParseStylesheetCallerParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
+xsltParseStylesheetCallerParam(xsltTransformContextPtr ctxt, xmlNodePtr cur)
+{
+#ifdef XSLT_REFACTORED
+    xsltStyleBasicItemVariablePtr comp;
+#else
+    xsltStylePreCompPtr comp;
+#endif
     xmlNodePtr tree = NULL;
     xsltStackElemPtr elem = NULL;
-    xsltStylePreCompPtr comp;
-
+    
     if ((cur == NULL) || (ctxt == NULL))
 	return(NULL);
+#ifdef XSLT_REFACTORED
+    comp = (xsltStyleBasicItemVariablePtr) cur->psvi;
+#else
     comp = (xsltStylePreCompPtr) cur->psvi;
+#endif     
     if (comp == NULL) {
 	xsltTransformError(ctxt, NULL, cur,
 	    "xsl:param : compilation error\n");
@@ -1379,7 +1442,7 @@ xsltParseStylesheetCallerParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
 	tree = cur;
     }
 
-    elem = xsltBuildVariable(ctxt, comp, tree);
+    elem = xsltBuildVariable(ctxt, (xsltStylePreCompPtr) comp, tree);
 
     return(elem);
 }
@@ -1394,14 +1457,23 @@ xsltParseStylesheetCallerParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
  */
 
 void
-xsltParseGlobalVariable(xsltStylesheetPtr style, xmlNodePtr cur) {
+xsltParseGlobalVariable(xsltStylesheetPtr style, xmlNodePtr cur)
+{
+#ifdef XSLT_REFACTORED
+    xsltStyleItemVariablePtr comp;
+#else
     xsltStylePreCompPtr comp;
+#endif
 
     if ((cur == NULL) || (style == NULL))
 	return;
 
     xsltStylePreCompute(style, cur);
+#ifdef XSLT_REFACTORED
+    comp = (xsltStyleItemVariablePtr) cur->psvi;
+#else
     comp = (xsltStylePreCompPtr) cur->psvi;
+#endif
     if (comp == NULL) {
 	xsltTransformError(NULL, style, cur,
 	     "xsl:variable : compilation failed\n");
@@ -1422,8 +1494,9 @@ xsltParseGlobalVariable(xsltStylesheetPtr style, xmlNodePtr cur) {
 	"Registering global variable %s\n", comp->name);
 #endif
 
-    xsltRegisterGlobalVariable(style, comp->name, comp->ns, comp->select,
-	                       cur->children, comp, NULL);
+    xsltRegisterGlobalVariable(style, comp->name, comp->ns,
+	comp->select, cur->children, (xsltStylePreCompPtr) comp,
+	NULL);
 }
 
 /**
@@ -1437,13 +1510,21 @@ xsltParseGlobalVariable(xsltStylesheetPtr style, xmlNodePtr cur) {
 
 void
 xsltParseGlobalParam(xsltStylesheetPtr style, xmlNodePtr cur) {
+#ifdef XSLT_REFACTORED
+    xsltStyleItemParamPtr comp;
+#else
     xsltStylePreCompPtr comp;
+#endif
 
     if ((cur == NULL) || (style == NULL))
 	return;
 
     xsltStylePreCompute(style, cur);
+#ifdef XSLT_REFACTORED
+    comp = (xsltStyleItemParamPtr) cur->psvi;
+#else
     comp = (xsltStylePreCompPtr) cur->psvi;
+#endif    
     if (comp == NULL) {
 	xsltTransformError(NULL, style, cur,
 	     "xsl:param : compilation failed\n");
@@ -1465,8 +1546,9 @@ xsltParseGlobalParam(xsltStylesheetPtr style, xmlNodePtr cur) {
 	"Registering global param %s\n", comp->name);
 #endif
 
-    xsltRegisterGlobalVariable(style, comp->name, comp->ns, comp->select,
-	                       cur->children, comp, NULL);
+    xsltRegisterGlobalVariable(style, comp->name, comp->ns,
+	comp->select, cur->children, (xsltStylePreCompPtr) comp,
+	NULL);
 }
 
 /**
@@ -1479,13 +1561,22 @@ xsltParseGlobalParam(xsltStylesheetPtr style, xmlNodePtr cur) {
  */
 
 void
-xsltParseStylesheetVariable(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
+xsltParseStylesheetVariable(xsltTransformContextPtr ctxt, xmlNodePtr cur)
+{
+#ifdef XSLT_REFACTORED
+    xsltStyleItemVariablePtr comp;
+#else
     xsltStylePreCompPtr comp;
+#endif
 
     if ((cur == NULL) || (ctxt == NULL))
 	return;
 
+#ifdef XSLT_REFACTORED
+    comp = (xsltStyleItemVariablePtr) cur->psvi;
+#else
     comp = (xsltStylePreCompPtr) cur->psvi;
+#endif   
     if (comp == NULL) {
 	xsltTransformError(ctxt, NULL, cur,
 	     "xsl:variable : compilation failed\n");
@@ -1503,7 +1594,7 @@ xsltParseStylesheetVariable(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
 	"Registering variable %s\n", comp->name));
 #endif
 
-    xsltRegisterVariable(ctxt, comp, cur->children, 0);
+    xsltRegisterVariable(ctxt, (xsltStylePreCompPtr) comp, cur->children, 0);
 }
 
 /**
@@ -1516,13 +1607,21 @@ xsltParseStylesheetVariable(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
  */
 
 void
-xsltParseStylesheetParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
+xsltParseStylesheetParam(xsltTransformContextPtr ctxt, xmlNodePtr cur)
+{
+#ifdef XSLT_REFACTORED
+    xsltStyleItemParamPtr comp;
+#else
     xsltStylePreCompPtr comp;
+#endif
 
     if ((cur == NULL) || (ctxt == NULL))
 	return;
-
+#ifdef XSLT_REFACTORED
+    comp = (xsltStyleItemParamPtr) cur->psvi;
+#else
     comp = (xsltStylePreCompPtr) cur->psvi;
+#endif 
     if (comp == NULL) {
 	xsltTransformError(ctxt, NULL, cur,
 	     "xsl:param : compilation failed\n");
@@ -1540,7 +1639,7 @@ xsltParseStylesheetParam(xsltTransformContextPtr ctxt, xmlNodePtr cur) {
 	"Registering param %s\n", comp->name));
 #endif
 
-    xsltRegisterVariable(ctxt, comp, cur->children, 1);
+    xsltRegisterVariable(ctxt, (xsltStylePreCompPtr) comp, cur->children, 1);
 }
 
 /**
