@@ -148,16 +148,28 @@ exclPrefixPop(xsltStylesheetPtr style)
  * Initializes the processor (e.g. registers built-in extensions,
  * etc.)
  */
-static void
-xsltInit (void) {
-    static int initialized = 0;
 
+static int initialized = 0;
+
+void
+xsltInit (void) {
     if (initialized == 0) {
 	initialized = 1;
-	xsltRegisterAllExtras();
+        xsltRegisterAllExtras();
     }
 }
 
+/**
+ * xsltUninit
+ *
+ * Uninitializes the processor.
+ */
+
+void
+xsltUninit (void) {
+    initialized = 0;
+}
+
 /**
  * xsltIsBlank:
  * @str:  a string
@@ -186,6 +198,8 @@ static xsltDecimalFormatPtr
 xsltNewDecimalFormat(xmlChar *name)
 {
     xsltDecimalFormatPtr self;
+    /* UTF-8 for 0x2030 */
+    static const xmlChar permille[4] = {0xe2, 0x80, 0xb0, 0};
 
     self = xmlMalloc(sizeof(xsltDecimalFormat));
     if (self != NULL) {
@@ -198,7 +212,7 @@ xsltNewDecimalFormat(xmlChar *name)
 	self->decimalPoint = xmlStrdup(BAD_CAST("."));
 	self->grouping = xmlStrdup(BAD_CAST(","));
 	self->percent = xmlStrdup(BAD_CAST("%"));
-	self->permille = xmlStrdup(BAD_CAST("?"));
+	self->permille = xmlStrdup(BAD_CAST(permille));
 	self->zeroDigit = xmlStrdup(BAD_CAST("0"));
 	self->minusSign = xmlStrdup(BAD_CAST("-"));
 	self->infinity = xmlStrdup(BAD_CAST("Infinity"));
@@ -355,6 +369,9 @@ xsltFreeTemplateList(xsltTemplatePtr template) {
 xsltStylesheetPtr
 xsltNewStylesheet(void) {
     xsltStylesheetPtr cur;
+#ifdef XSLT_REFACTORED
+    xsltCompilerCtxtPtr cctxt;
+#endif
 
     cur = (xsltStylesheetPtr) xmlMalloc(sizeof(xsltStylesheet));
     if (cur == NULL) {
@@ -363,6 +380,26 @@ xsltNewStylesheet(void) {
 	return(NULL);
     }
     memset(cur, 0, sizeof(xsltStylesheet));
+#ifdef XSLT_REFACTORED
+    /*
+    * Create the compiler context.
+    */
+    cctxt = (xsltCompilerCtxtPtr) xmlMalloc(sizeof(xsltCompilerCtxt));
+    if (cctxt == NULL) {
+	xmlFree(cur);
+	xsltTransformError(NULL, NULL, NULL,
+		"xsltNewStylesheet : malloc of compilation context failed\n");
+	return(NULL);
+    }
+    memset(cctxt, 0, sizeof(xsltCompilerCtxt));
+    cur->compCtxt = (void *) cctxt;
+    cctxt->sheet = cur;
+#endif
+    /*
+    * TODO: This here seems to be the best place where to create
+    *   the compilation context, right?
+    */
+    	    
     cur->omitXmlDeclaration = -1;
     cur->standalone = -1;
     cur->decimalFormat = xsltNewDecimalFormat(NULL);
@@ -375,6 +412,7 @@ xsltNewStylesheet(void) {
     cur->extInfos = NULL;
     cur->extrasNr = 0;
     cur->internalized = 1;
+    cur->literal_result = 0;
     cur->dict = xmlDictCreate();
 #ifdef WITH_XSLT_DEBUG
     xsltGenericDebug(xsltGenericDebugContext,
@@ -472,6 +510,58 @@ xsltFreeStylesheetList(xsltStylesheetPtr sheet) {
     }
 }
 
+#ifdef XSLT_REFACTORED
+/**
+ * xsltFreeInScopeNamespaces:
+ * @sheet:  an XSLT stylesheet
+ *
+ * Frees the list of in-scope namespace lists.
+ */
+static void
+xsltFreeInScopeNamespaces(xsltStylesheetPtr sheet)
+{
+    if (sheet->inScopeNamespaces == NULL)
+	return;
+    else {
+	int i;
+	xsltNsListPtr nsi;
+	xsltPointerListPtr list = sheet->inScopeNamespaces;
+
+	for (i = 0; i < list->number; i++) {
+	    /*
+	    * REVISIT TODO: Free info of in-scope namespaces.
+	    */
+	    nsi = (xsltNsListPtr) list->items[i];
+	    if (nsi->list != NULL)
+		xmlFree(nsi->list);
+	    xmlFree(nsi);
+	}
+	xsltPointerListFree(list);
+	sheet->inScopeNamespaces = NULL;
+    }
+    return;
+}
+
+static void
+xsltCompilerCtxtFree(xsltCompilerCtxtPtr cctxt)
+{    
+    if (cctxt == NULL)
+	return;
+    /*
+    * Free node-infos.
+    */
+    if (cctxt->inodeList != NULL) {
+	xsltCompilerNodeInfoPtr tmp, cur = cctxt->inodeList;
+	while (cur != NULL) {
+	    tmp = cur;
+	    cur = cur->next;
+	    xmlFree(tmp);
+	}
+    }
+    xmlFree(cctxt);
+}
+#endif
+
 /**
  * xsltFreeStylesheet:
  * @sheet:  an XSLT stylesheet
@@ -491,6 +581,10 @@ xsltFreeStylesheet(xsltStylesheetPtr sheet)
     xsltFreeTemplateList(sheet->templates);
     xsltFreeAttributeSetsHashes(sheet);
     xsltFreeNamespaceAliasHashes(sheet);
+#ifdef XSLT_REFACTORED
+    if (sheet->inScopeNamespaces != NULL)
+	xsltFreeInScopeNamespaces(sheet);
+#endif
     xsltFreeStyleDocuments(sheet);
     xsltFreeStylePreComps(sheet);
     xsltShutdownExts(sheet);
@@ -526,6 +620,15 @@ xsltFreeStylesheet(xsltStylesheetPtr sheet)
 
     if (sheet->imports != NULL)
         xsltFreeStylesheetList(sheet->imports);
+#ifdef XSLT_REFACTORED
+    /*
+    * TODO: Just a paranoid cleanup, in case the compilation
+    *   context was not freed after the compilation.
+    */
+    if (sheet->compCtxt != NULL) {
+	xsltCompilerCtxtFree((xsltCompilerCtxtPtr) sheet->compCtxt);
+    }
+#endif
 
 #ifdef WITH_XSLT_DEBUG
     xsltGenericDebug(xsltGenericDebugContext,
@@ -566,6 +669,16 @@ xsltGetInheritedNsList(xsltStylesheetPtr style,
     int maxns = 10;
     int i;
 
+    /*
+    * TODO: This will gather the ns-decls of elements even if
+    * outside xsl:stylesheet. Example:
+    * <doc xmlns:foo="urn:test:foo">
+    *  	<xsl:stylesheet ...    
+    * </doc>
+    * Will have foo="urn:test:foo" in the list.
+    * Is this a bug?
+    */
+
     if ((style == NULL) || (template == NULL) || (node == NULL) ||
 	(template->inheritedNsNr != 0) || (template->inheritedNs != NULL))
 	return(0);
@@ -578,6 +691,11 @@ xsltGetInheritedNsList(xsltStylesheetPtr style,
 		if ((cur->prefix != NULL) &&
 		    (xsltCheckExtPrefix(style, cur->prefix)))
 		    goto skip_ns;
+		/*
+		* Check if this namespace was excluded.
+		* Note that at this point only the exclusions defined
+		* on the topmost stylesheet element are in the exclusion-list.
+		*/
 		for (i = 0;i < style->exclPrefixNr;i++) {
 		    if (xmlStrEqual(cur->href, style->exclPrefixTab[i]))
 			goto skip_ns;
@@ -593,6 +711,9 @@ xsltGetInheritedNsList(xsltStylesheetPtr style,
                     }
                     ret[nbns] = NULL;
                 }
+		/*
+		* Skip shadowed namespace bindings.
+		*/
                 for (i = 0; i < nbns; i++) {
                     if ((cur->prefix == ret[i]->prefix) ||
                         (xmlStrEqual(cur->prefix, ret[i]->prefix)))
@@ -650,24 +771,25 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
 
     if ((cur == NULL) || (style == NULL))
         return;
-
-    prop = xsltGetNsProp(cur, (const xmlChar *) "version", XSLT_NAMESPACE);
+   
+    prop = xmlGetNsProp(cur, (const xmlChar *) "version", NULL);
     if (prop != NULL) {
         if (style->version != NULL)
             xmlFree(style->version);
         style->version = prop;
     }
 
-    prop =
-        xsltGetNsProp(cur, (const xmlChar *) "encoding", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *) "encoding", NULL);
     if (prop != NULL) {
         if (style->encoding != NULL)
             xmlFree(style->encoding);
         style->encoding = prop;
     }
 
-    /* relaxed to support xt:document */
-    prop = xmlGetProp(cur, (const xmlChar *) "method");
+    /* relaxed to support xt:document
+    * TODO KB: What does "relaxed to support xt:document" mean?
+    */
+    prop = xmlGetNsProp(cur, (const xmlChar *) "method", NULL);
     if (prop != NULL) {
         const xmlChar *URI;
 
@@ -697,26 +819,21 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
 	}
     }
 
-    prop =
-        xsltGetNsProp(cur, (const xmlChar *) "doctype-system",
-                      XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *) "doctype-system", NULL);
     if (prop != NULL) {
         if (style->doctypeSystem != NULL)
             xmlFree(style->doctypeSystem);
         style->doctypeSystem = prop;
     }
 
-    prop =
-        xsltGetNsProp(cur, (const xmlChar *) "doctype-public",
-                      XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *) "doctype-public", NULL);
     if (prop != NULL) {
         if (style->doctypePublic != NULL)
             xmlFree(style->doctypePublic);
         style->doctypePublic = prop;
     }
 
-    prop = xsltGetNsProp(cur, (const xmlChar *) "standalone",
-                         XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *) "standalone", NULL);
     if (prop != NULL) {
         if (xmlStrEqual(prop, (const xmlChar *) "yes")) {
             style->standalone = 1;
@@ -730,7 +847,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         xmlFree(prop);
     }
 
-    prop = xsltGetNsProp(cur, (const xmlChar *) "indent", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *) "indent", NULL);
     if (prop != NULL) {
         if (xmlStrEqual(prop, (const xmlChar *) "yes")) {
             style->indent = 1;
@@ -744,8 +861,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         xmlFree(prop);
     }
 
-    prop = xsltGetNsProp(cur, (const xmlChar *) "omit-xml-declaration",
-                         XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *) "omit-xml-declaration", NULL);
     if (prop != NULL) {
         if (xmlStrEqual(prop, (const xmlChar *) "yes")) {
             style->omitXmlDeclaration = 1;
@@ -760,9 +876,8 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         xmlFree(prop);
     }
 
-    elements =
-        xsltGetNsProp(cur, (const xmlChar *) "cdata-section-elements",
-                      XSLT_NAMESPACE);
+    elements = xmlGetNsProp(cur, (const xmlChar *) "cdata-section-elements",
+	NULL);
     if (elements != NULL) {
         if (style->cdataSection == NULL)
             style->cdataSection = xmlHashCreate(10);
@@ -814,7 +929,7 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
         xmlFree(elements);
     }
 
-    prop = xsltGetNsProp(cur, (const xmlChar *) "media-type", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *) "media-type", NULL);
     if (prop != NULL) {
 	if (style->mediaType)
 	    xmlFree(style->mediaType);
@@ -827,6 +942,13 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
  * @style:  the XSLT stylesheet
  * @cur:  the "decimal-format" element
  *
+ * <!-- Category: top-level-element -->
+ * <xsl:decimal-format
+ *   name = qname, decimal-separator = char, grouping-separator = char,
+ *   infinity = string, minus-sign = char, NaN = string, percent = char
+ *   per-mille = char, zero-digit = char, digit = char,
+ * pattern-separator = char />
+ *
  * parse an XSLT stylesheet decimal-format element and
  * and record the formatting characteristics
  */
@@ -842,7 +964,7 @@ xsltParseStylesheetDecimalFormat(xsltStylesheetPtr style, xmlNodePtr cur)
 
     format = style->decimalFormat;
     
-    prop = xsltGetNsProp(cur, BAD_CAST("name"), XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, BAD_CAST("name"), NULL);
     if (prop != NULL) {
 	format = xsltDecimalFormatGetByName(style, prop);
 	if (format != NULL) {
@@ -865,64 +987,61 @@ xsltParseStylesheetDecimalFormat(xsltStylesheetPtr style, xmlNodePtr cur)
 	    iter->next = format;
     }
 
-    prop = xsltGetNsProp(cur, (const xmlChar *)"decimal-separator",
-	                XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"decimal-separator", NULL);
     if (prop != NULL) {
 	if (format->decimalPoint != NULL) xmlFree(format->decimalPoint);
 	format->decimalPoint  = prop;
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"grouping-separator",
-	                XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"grouping-separator", NULL);
     if (prop != NULL) {
 	if (format->grouping != NULL) xmlFree(format->grouping);
 	format->grouping  = prop;
     }
 
-    prop = xsltGetNsProp(cur, (const xmlChar *)"infinity", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"infinity", NULL);
     if (prop != NULL) {
 	if (format->infinity != NULL) xmlFree(format->infinity);
 	format->infinity  = prop;
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"minus-sign", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"minus-sign", NULL);
     if (prop != NULL) {
 	if (format->minusSign != NULL) xmlFree(format->minusSign);
 	format->minusSign  = prop;
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"NaN", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"NaN", NULL);
     if (prop != NULL) {
 	if (format->noNumber != NULL) xmlFree(format->noNumber);
 	format->noNumber  = prop;
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"percent", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"percent", NULL);
     if (prop != NULL) {
 	if (format->percent != NULL) xmlFree(format->percent);
 	format->percent  = prop;
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"per-mille", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"per-mille", NULL);
     if (prop != NULL) {
 	if (format->permille != NULL) xmlFree(format->permille);
 	format->permille  = prop;
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"zero-digit", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"zero-digit", NULL);
     if (prop != NULL) {
 	if (format->zeroDigit != NULL) xmlFree(format->zeroDigit);
 	format->zeroDigit  = prop;
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"digit", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"digit", NULL);
     if (prop != NULL) {
 	if (format->digit != NULL) xmlFree(format->digit);
 	format->digit  = prop;
     }
     
-    prop = xsltGetNsProp(cur, (const xmlChar *)"pattern-separator",
-	                XSLT_NAMESPACE);
+    prop = xmlGetNsProp(cur, (const xmlChar *)"pattern-separator", NULL);
     if (prop != NULL) {
 	if (format->patternSeparator != NULL) xmlFree(format->patternSeparator);
 	format->patternSeparator  = prop;
@@ -946,7 +1065,7 @@ xsltParseStylesheetPreserveSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
     if ((cur == NULL) || (style == NULL))
 	return;
 
-    elements = xsltGetNsProp(cur, (const xmlChar *)"elements", XSLT_NAMESPACE);
+    elements = xmlGetNsProp(cur, (const xmlChar *)"elements", NULL);
     if (elements == NULL) {
 	xsltTransformError(NULL, style, cur,
 	    "xsltParseStylesheetPreserveSpace: missing elements attribute\n");
@@ -994,20 +1113,34 @@ xsltParseStylesheetPreserveSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
  * @style:  the XSLT stylesheet
  * @template:  the "extension-element-prefixes" prefix
  *
- * parse an XSLT stylesheet extension prefix and record
- * prefixes needing stripping
+ * parse an XSLT stylesheet's "extension-element-prefix" attribute value
+ * and register the namespaces of extension elements.
+ * SPEC "A namespace is designated as an extension namespace by using
+ *   an extension-element-prefixes attribute on:
+ *   1) an xsl:stylesheet element
+ *   2) TODO: an xsl:extension-element-prefixes attribute on a
+ *      literal result element 
+ *   3) TODO: an extension element."
  */
 
 static void
-xsltParseStylesheetExtPrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
+xsltParseStylesheetExtPrefix(xsltStylesheetPtr style, xmlNodePtr cur,
+			     int isXsltElem) {
     xmlChar *prefixes;
     xmlChar *prefix, *end;
 
     if ((cur == NULL) || (style == NULL))
 	return;
 
-    prefixes = xsltGetNsProp(cur, (const xmlChar *)"extension-element-prefixes",
-	                    XSLT_NAMESPACE);
+    if (isXsltElem) {
+	/* For xsl:stylesheet/xsl:transform. */
+	prefixes = xmlGetNsProp(cur,
+	    (const xmlChar *)"extension-element-prefixes", NULL);
+    } else {
+	/* For literal result elements and extension elements. */
+	prefixes = xmlGetNsProp(cur,
+	    (const xmlChar *)"extension-element-prefixes", XSLT_NAMESPACE);
+    }
     if (prefixes == NULL) {
 	return;
     }
@@ -1051,8 +1184,8 @@ xsltParseStylesheetExtPrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
  * @style:  the XSLT stylesheet
  * @cur:  the "strip-space" element
  *
- * parse an XSLT stylesheet strip-space element and record
- * elements needing stripping
+ * parse an XSLT stylesheet's strip-space element and record
+ * the elements needing stripping
  */
 
 static void
@@ -1063,7 +1196,7 @@ xsltParseStylesheetStripSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
     if ((cur == NULL) || (style == NULL))
 	return;
 
-    elements = xsltGetNsProp(cur, (const xmlChar *)"elements", XSLT_NAMESPACE);
+    elements = xmlGetNsProp(cur, (const xmlChar *)"elements", NULL);
     if (elements == NULL) {
 	xsltTransformError(NULL, style, cur,
 	    "xsltParseStylesheetStripSpace: missing elements attribute\n");
@@ -1118,7 +1251,9 @@ xsltParseStylesheetStripSpace(xsltStylesheetPtr style, xmlNodePtr cur) {
  */
 
 static int
-xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
+xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur,
+				 int isXsltElem)
+{
     int nb = 0;
     xmlChar *prefixes;
     xmlChar *prefix, *end;
@@ -1126,8 +1261,13 @@ xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
     if ((cur == NULL) || (style == NULL))
 	return(0);
 
-    prefixes = xsltGetNsProp(cur, (const xmlChar *)"exclude-result-prefixes",
-	                    XSLT_NAMESPACE);
+    if (isXsltElem)
+	prefixes = xmlGetNsProp(cur,
+	    (const xmlChar *)"exclude-result-prefixes", NULL);
+    else
+	prefixes = xmlGetNsProp(cur,
+	    (const xmlChar *)"exclude-result-prefixes", XSLT_NAMESPACE);
+
     if (prefixes == NULL) {
 	return(0);
     }
@@ -1168,6 +1308,74 @@ xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
     return(nb);
 }
 
+#ifdef XSLT_REFACTORED
+
+static xsltCompilerNodeInfoPtr
+xsltCompilerNodePush(xsltCompilerCtxtPtr cctxt, xmlNodePtr node)
+{    
+    if ((cctxt->inode != NULL) && (cctxt->inode->next != NULL)) {	
+	cctxt->inode = cctxt->inode->next;
+    } else if ((cctxt->inode == NULL) && (cctxt->inodeList != NULL)) {
+	cctxt->inode = cctxt->inodeList;	
+    } else {
+	/*
+	* Create a new node-info.
+	*/
+	cctxt->inode = (xsltCompilerNodeInfoPtr)
+	    xmlMalloc(sizeof(xsltCompilerNodeInfo));
+	if (cctxt->inode == NULL) {
+	    xsltTransformError(NULL, cctxt->sheet, NULL,
+		"xsltCompilerNodePush: malloc failed.\n");
+	    return(NULL);
+	}
+	memset(cctxt->inode, 0, sizeof(xsltCompilerNodeInfo));
+	if (cctxt->inodeList == NULL)
+	    cctxt->inodeList = cctxt->inode;
+	else {
+	    cctxt->inodeLast->next = cctxt->inode;
+	    cctxt->inode->prev = cctxt->inodeLast;
+	}
+	cctxt->inodeLast = cctxt->inode;
+    }
+    /*
+    * REVISIT TODO: Keep the reset always complete.
+    */
+    cctxt->depth++;
+    cctxt->inode->depth = cctxt->depth;
+    cctxt->inode->templ = NULL;
+    cctxt->inode->item = NULL;
+    cctxt->inode->node = node;
+    /*
+    * Inherit the list of in-scope namespaces.
+    */
+    if (cctxt->inode->prev != NULL)
+	cctxt->inode->inScopeNS = cctxt->inode->prev->inScopeNS;
+    else
+	cctxt->inode->inScopeNS = NULL;
+    
+    return(cctxt->inode);
+}
+
+static void
+xsltCompilerNodePop(xsltCompilerCtxtPtr cctxt, xmlNodePtr node)
+{    
+    if (cctxt->inode == NULL) {
+	xmlGenericError(xmlGenericErrorContext,
+	    "xsltCompilerNodePop: Top-node mismatch.\n");
+	return;
+    }    	
+    if (cctxt->inode->node != node)
+	xmlGenericError(xmlGenericErrorContext,
+	"xsltCompilerNodePop: Node mismatch.\n");
+    if (cctxt->inode->depth != cctxt->depth)
+	xmlGenericError(xmlGenericErrorContext,
+	"xsltCompilerNodePop: Depth mismatch.\n");
+    cctxt->depth--;
+    cctxt->inode = cctxt->inode->prev;
+}
+
+#endif
+
 /**
  * xsltPrecomputeStylesheet:
  * @style:  the XSLT stylesheet
@@ -1177,18 +1385,35 @@ xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
  * and run the preprocessing of all XSLT constructs.
  *
  * and process xslt:text
+ *
+ * URGENT TODO: In order to avoid separation of the parsing of the stylesheet's
+ *   node-tree, this should either only strip whitespace-only text-nodes,
+ *   or it should be merged completely with the stylesheet-parsing
+ *   functions (e.g. xsltParseStylesheetTop()).
  */
 static void
 xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
     xmlNodePtr delete;
     int internalize = 0;
+#ifdef XSLT_REFACTORED
+    xsltCompilerCtxtPtr cctxt;
+#endif
 
-    if ((style != NULL) && (cur != NULL) && (cur->doc != NULL) &&
-        (style->dict != NULL) && (cur->doc->dict == style->dict))
+    if ((style == NULL) || (cur == NULL)
+#ifdef XSLT_REFACTORED
+	||(style->compCtxt == NULL)
+#endif
+	)
+        return;
+
+    if ((cur->doc != NULL) && (style->dict != NULL) &&
+        (cur->doc->dict == style->dict))
 	internalize = 1;
     else
         style->internalized = 0;
-
+#ifdef XSLT_REFACTORED
+    cctxt = (xsltCompilerCtxtPtr) style->compCtxt;    
+#endif
     /*
      * This content comes from the stylesheet
      * For stylesheets, the set of whitespace-preserving
@@ -1206,8 +1431,11 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    delete = NULL;
 	}
 	if (cur->type == XML_ELEMENT_NODE) {
-	    int exclPrefixes;
+	    int exclPrefixes;	    
 
+#ifdef XSLT_REFACTORED
+	    xsltCompilerNodePush(cctxt, cur);
+#endif
 	    /*
 	     * Internalize attributes values.
 	     */
@@ -1220,34 +1448,74 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 		    if ((txt != NULL) && (txt->type == XML_TEXT_NODE) &&
 		        (txt->content != NULL) &&
 			(!xmlDictOwns(style->dict, txt->content))) {
-			xmlChar *old = (xmlChar *) txt->content;
+			xmlChar *tmp;
 
 			/*
 			 * internalize the text string, goal is to speed
 			 * up operations and minimize used space by compiled
 			 * stylesheets.
 			 */
-			txt->content = (xmlChar *)
-			               xmlDictLookup(style->dict, old, -1);
-			if (old != txt->content)
-			    xmlFree(old);
+			tmp = (xmlChar *) xmlDictLookup(style->dict,
+			                                txt->content, -1);
+			if (tmp != txt->content) {
+			    xmlNodeSetContent(txt, NULL);
+			    txt->content = tmp;
+			}
 		    }
 		    prop = prop->next;
 		}
 	    }
-
-	    exclPrefixes = xsltParseStylesheetExcludePrefix(style, cur);
+	    /*
+	    * TODO: "exclude-result-prefixes"
+	    *   SPEC 1.0:
+	    *   "exclude-result-prefixes" is only allowed on literal
+	    *   result elements and "xsl:exclude-result-prefixes" only
+	    *   on xsl:stylesheet/xsl:transform.
+	    *   SPEC 2.0:
+	    *   "There are a number of standard attributes
+	    *   that may appear on any XSLT element: specifically version,
+	    *   exclude-result-prefixes, extension-element-prefixes,
+	    *   xpath-default-namespace, default-collation, and use-when."
+	    */
 	    if (IS_XSLT_ELEM(cur)) {
+		exclPrefixes = 0;
+#ifdef XSLT_REFACTORED
+		if ((cctxt->depth == 0) && (cur->nsDef != NULL)) {
+		    /*
+		    * In every case, we need the in-scope namespaces of the
+		    * element, where the stylesheet is rooted at.
+		    * Otherwise we need to pre-compute the in-scope namespaces
+		    * only if there's a new ns-decl.
+		    */
+		    cctxt->inode->inScopeNS =
+			xsltCompilerGetInScopeNSInfo(cctxt, cur);
+		}
+#endif		
 		xsltStylePreCompute(style, cur);
 		if (IS_XSLT_NAME(cur, "text")) {
 		    for (;exclPrefixes > 0;exclPrefixes--)
 			exclPrefixPop(style);
 		    goto skip_children;
 		}
+	    } else {
+#ifdef XSLT_REFACTORED
+		if (cctxt->depth == 0)
+		    cctxt->inode->inScopeNS =
+			xsltCompilerGetInScopeNSInfo(cctxt, cur);
+#endif
+		exclPrefixes = xsltParseStylesheetExcludePrefix(style, cur, 0);
 	    }
 
 	    /*
 	     * Remove excluded prefixes
+	     * TODO BUG:
+	     *   This will incorrectly apply excluded-result-prefixes
+	     *   of the including stylesheet to the included stylesheet.
+	     *   We need to localize the list of excluded-prefixes for
+	     *   every processed stylesheet.
+	     * SPEC 1.0: "a subtree rooted at an xsl:stylesheet element
+	     *   does not include any stylesheets imported or included by
+	     *   children of that xsl:stylesheet element."
 	     */
 	    if ((cur->nsDef != NULL) && (style->exclPrefixNr > 0)) {
 		xmlNsPtr ns = cur->nsDef, prev = NULL, next;
@@ -1304,15 +1572,16 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 		}
 	    } else if ((cur->content != NULL) && (internalize) &&
 	               (!xmlDictOwns(style->dict, cur->content))) {
-		xmlChar *old = (xmlChar *) cur->content;
+		xmlChar *tmp;
 
 		/*
 		 * internalize the text string, goal is to speed
 		 * up operations and minimize used space by compiled
 		 * stylesheets.
 		 */
-		cur->content = (xmlChar *) xmlDictLookup(style->dict, old, -1);
-		xmlFree(old);
+		tmp = (xmlChar *) xmlDictLookup(style->dict, cur->content, -1);
+		xmlNodeSetContent(cur, NULL);
+		cur->content = tmp;
 	    }
 	} else if ((cur->type != XML_ELEMENT_NODE) &&
 		   (cur->type != XML_CDATA_SECTION_NODE)) {
@@ -1331,13 +1600,20 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 		continue;
 	    }
 	}
+#ifdef XSLT_REFACTORED
+	if (cur->type == XML_ELEMENT_NODE) {
+	    /* Leaving the scope of an element-node. */
+	    xsltCompilerNodePop(cctxt, cur);
+	}
+#endif
+
 skip_children:
 	if (cur->next != NULL) {
 	    cur = cur->next;
 	    continue;
-	}
-	
+	}	
 	do {
+
 	    cur = cur->parent;
 	    if (cur == NULL)
 		break;
@@ -1375,11 +1651,20 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
     xmlNodePtr cur;
     const xmlChar *URI;
 
+    if (style == NULL)
+        return;
     /* 
      * TODO: basically if the stylesheet uses the same prefix for different
      *       patterns, well they may be in problem, hopefully they will get
      *       a warning first.
      */
+    /*
+    * TODO: Eliminate the use of the hash for XPath expressions.
+    *   An expression should be evaluated in the context of the in-scope
+    *   namespaces; eliminate the restriction of an XML document to contain
+    *   no duplicate prefixes for different namespace names.
+    * 
+    */
     cur = xmlDocGetRootElement(style->doc);
     while (cur != NULL) {
 	if (cur->type == XML_ELEMENT_NODE) {
@@ -1391,7 +1676,7 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 			if (style->nsHash == NULL) {
 			    xsltTransformError(NULL, style, cur,
 		 "xsltGatherNamespaces: failed to create hash table\n");
-			    if (style != NULL) style->errors++;
+			    style->errors++;
 			    return;
 			}
 		    }
@@ -1399,7 +1684,7 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 		    if ((URI != NULL) && (!xmlStrEqual(URI, ns->href))) {
 			xsltTransformError(NULL, style, cur,
 	     "Namespaces prefix %s used for multiple namespaces\n",ns->prefix);
-			if (style != NULL) style->warnings++;
+			style->warnings++;
 		    } else if (URI == NULL) {
 			xmlHashUpdateEntry(style->nsHash, ns->prefix,
 			    (void *) ns->href, (xmlHashDeallocator)xmlFree);
@@ -1481,9 +1766,9 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 		    xmlNodePtr text = cur->children, next;
 		    int noesc = 0;
 			
-		    prop = xsltGetNsProp(cur,
-			    (const xmlChar *)"disable-output-escaping",
-					XSLT_NAMESPACE);
+		    prop = xmlGetNsProp(cur,
+			(const xmlChar *)"disable-output-escaping",
+			NULL);
 		    if (prop != NULL) {
 #ifdef WITH_XSLT_DEBUG_PARSING
 			xsltGenericDebug(xsltGenericDebugContext,
@@ -1495,7 +1780,7 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 						(const xmlChar *)"no")){
 			    xsltTransformError(NULL, style, cur,
 	     "xsl:text: disable-output-escaping allows only yes or no\n");
-			    if (style != NULL) style->warnings++;
+			    style->warnings++;
 
 			}
 			xmlFree(prop);
@@ -1510,7 +1795,7 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 			     (text->type != XML_CDATA_SECTION_NODE)) {
 			    xsltTransformError(NULL, style, cur,
 		 "xsltParseTemplateContent: xslt:text content problem\n");
-			    if (style != NULL) style->errors++;
+			    style->errors++;
 			    break;
 			}
 			if ((noesc) && (text->type != XML_CDATA_SECTION_NODE))
@@ -1522,26 +1807,24 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 		     * replace xsl:text by the list of childs
 		     */
 		    if (text == NULL) {
-		        int internalize = 0;
-			if ((style != NULL) && (text != NULL) &&
-			    (text->doc != NULL) && (style->dict != NULL) &&
-			    (text->doc->dict == style->dict))
-			    internalize = 1;
-			else
-			    style->internalized = 0;
-
 			text = cur->children;
 			while (text != NULL) {
 			    if ((text->content != NULL) &&
 			        (!xmlDictOwns(style->dict, text->content))) {
-				xmlChar *old = (xmlChar *) text->content;
 
 				/*
 				 * internalize the text string
 				 */
-				text->content = (xmlChar *)
-					xmlDictLookup(style->dict, old, -1);
-				xmlFree(old);
+				if (text->doc->dict != NULL) {
+				    const xmlChar *tmp;
+				    
+				    tmp = xmlDictLookup(text->doc->dict,
+				                        text->content, -1);
+				    if (tmp != text->content) {
+				        xmlNodeSetContent(text, NULL);
+					text->content = (xmlChar *) tmp;
+				    }
+				}
 			    }
 
 			    next = text->next;
@@ -1652,6 +1935,9 @@ skip_children:
  * @style:  the XSLT stylesheet
  * @key:  the "key" element
  *
+ * <!-- Category: top-level-element -->
+ * <xsl:key name = qname, match = pattern, use = expression />
+ *
  * parse an XSLT stylesheet key definition and register it
  */
 
@@ -1669,7 +1955,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
     /*
      * Get arguments
      */
-    prop = xsltGetNsProp(key, (const xmlChar *)"name", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(key, (const xmlChar *)"name", NULL);
     if (prop != NULL) {
         const xmlChar *URI;
 
@@ -1693,7 +1979,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 	goto error;
     }
 
-    match = xsltGetNsProp(key, (const xmlChar *)"match", XSLT_NAMESPACE);
+    match = xmlGetNsProp(key, (const xmlChar *)"match", NULL);
     if (match == NULL) {
 	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing match\n");
@@ -1701,7 +1987,7 @@ xsltParseStylesheetKey(xsltStylesheetPtr style, xmlNodePtr key) {
 	goto error;
     }
 
-    use = xsltGetNsProp(key, (const xmlChar *)"use", XSLT_NAMESPACE);
+    use = xmlGetNsProp(key, (const xmlChar *)"use", NULL);
     if (use == NULL) {
 	xsltTransformError(NULL, style, key,
 	    "xsl:key : error missing use\n");
@@ -1741,7 +2027,6 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     xmlChar *mode = NULL;
     xmlChar *modeURI = NULL;
     double  priority;
-    int exclPrefixes;
 
     if (template == NULL)
 	return;
@@ -1755,21 +2040,20 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     ret->next = style->templates;
     style->templates = ret;
     ret->style = style;
-
-    /*
-     * Check excluded prefixes
-     */
-    exclPrefixes = xsltParseStylesheetExcludePrefix(style, template);
-
+   
     /*
      * Get inherited namespaces
      */
+    /*
+    * TODO: Apply the optimized in-scope-namespace mechanism
+    *   as for the other XSLT instructions.
+    */
     xsltGetInheritedNsList(style, ret, template);
 
     /*
      * Get arguments
      */
-    prop = xsltGetNsProp(template, (const xmlChar *)"mode", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(template, (const xmlChar *)"mode", NULL);
     if (prop != NULL) {
         const xmlChar *URI;
 
@@ -1791,20 +2075,20 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
         if (mode != NULL) xmlFree(mode);
 	if (modeURI != NULL) xmlFree(modeURI);
     }
-    prop = xsltGetNsProp(template, (const xmlChar *)"match", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(template, (const xmlChar *)"match", NULL);
     if (prop != NULL) {
 	if (ret->match != NULL) xmlFree(ret->match);
 	ret->match  = prop;
     }
 
-    prop = xsltGetNsProp(template, (const xmlChar *)"priority", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(template, (const xmlChar *)"priority", NULL);
     if (prop != NULL) {
 	priority = xmlXPathStringEvalNumber(prop);
 	ret->priority = (float) priority;
 	xmlFree(prop);
     }
 
-    prop = xsltGetNsProp(template, (const xmlChar *)"name", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(template, (const xmlChar *)"name", NULL);
     if (prop != NULL) {
         const xmlChar *URI;
 	xsltTemplatePtr cur;
@@ -1859,18 +2143,96 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     xsltAddTemplate(style, ret, ret->mode, ret->modeURI);
 
 error:
-    for (;exclPrefixes > 0;exclPrefixes--)
-	exclPrefixPop(style);
+    return;
 }
 
+#ifdef XSLT_REFACTORED_PARSING
+/*
+* xsltParseStylesheetTreeNew:
+*
+* Parses and compiles an XSLT stylesheet's XML tree.
+*
+* TODO: Adjust error report text.
+*/
+static int
+xsltParseStylesheetTreeNew(xsltStylesheetPtr sheet, xmlNodePtr cur)
+{
+    xsltCompilerCtxtPtr cctxt;
+    xmlNodePtr top = cur;
+    int depth = 0, simpleSyntax = 0;
+
+    if ((cur == NULL) || (cur->type != XML_ELEMENT_NODE) ||
+	(sheet == NULL) || (sheet->compCtxt == NULL))
+	return(-1);
+    cctxt = (xsltCompilerCtxtPtr) sheet->compCtxt;
+
+    /*
+    * Evalute if we have a simplified syntax.
+    */
+    if ((IS_XSLT_ELEM(cur)) && 
+	((IS_XSLT_NAME(cur, "stylesheet")) ||
+	 (IS_XSLT_NAME(cur, "transform"))))
+    {	
+#ifdef WITH_XSLT_DEBUG_PARSING
+	xsltGenericDebug(xsltGenericDebugContext,
+		"xsltParseStylesheetProcess : found stylesheet\n");
+#endif
+	/*
+	* TODO: Initialize.
+	*/
+    } else {
+	simpleSyntax = 1;	
+	/*
+	* TODO: Create the initial template.
+	* TODO: Initialize.
+	*/
+    }        
+    /*
+    * Reset the compiler context.
+    */
+    cctxt->depth = 0;
+    cctxt->inode = NULL;
+    /*
+    * Initialize the in-scope namespaces. We need to have
+    * the in-scope ns-decls of the first given node, regardless
+    * if it's a XSLT instruction or a literal result element.
+    */
+    xsltCompilerNodePush(cctxt, cur);
+    xsltCompilerGetInScopeNSInfo(cctxt, cur);
+    goto next_sibling;
+
+    while (cur != NULL) {
+	if (cur->type == XML_ELEMENT_NODE) {
+	    
+	}
+
+next_sibling:
+	if (cur->type == XML_ELEMENT_NODE) {
+	    /*
+	    * Leaving an element.
+	    */
+	    xsltCompilerNodePop(cctxt, cur);
+	}
+	if (cur == top)
+	    break;
+	if (cur->next != NULL)
+	    cur = cur->next;
+	else {
+	    cur = cur->parent;
+	    goto next_sibling;
+	}	
+    }
+    return(0);
+}
+#endif /* XSLT_REFACTORED_PARSING */
+
 /**
  * xsltParseStylesheetTop:
  * @style:  the XSLT stylesheet
- * @top:  the top level "stylesheet" element
+ * @top:  the top level "stylesheet" or "transform" element
  *
  * scan the top level elements of an XSL stylesheet
  */
-
 static void
 xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
     xmlNodePtr cur;
@@ -1882,7 +2244,7 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
     if (top == NULL)
 	return;
 
-    prop = xsltGetNsProp(top, (const xmlChar *)"version", XSLT_NAMESPACE);
+    prop = xmlGetNsProp(top, (const xmlChar *)"version", NULL);
     if (prop == NULL) {
 	xsltTransformError(NULL, style, top,
 	    "xsl:version is missing: document may not be a stylesheet\n");
@@ -1898,8 +2260,6 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
 	xmlFree(prop);
     }
 
-    xsltParseStylesheetExtPrefix(style, top);
-
     cur = top->children;
 
     /*
@@ -2043,19 +2403,40 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	xsltTransformError(NULL, ret, (xmlNodePtr) doc,
 		"xsltParseStylesheetProcess : empty stylesheet\n");
 	return(NULL);
-    }
-    xsltParseStylesheetExcludePrefix(ret, cur);
-    if (!ret->nopreproc)
-	xsltPrecomputeStylesheet(ret, cur);
-
+    }    
     if ((IS_XSLT_ELEM(cur)) && 
 	((IS_XSLT_NAME(cur, "stylesheet")) ||
-	 (IS_XSLT_NAME(cur, "transform")))) {
+	 (IS_XSLT_NAME(cur, "transform")))) {	
 #ifdef WITH_XSLT_DEBUG_PARSING
 	xsltGenericDebug(xsltGenericDebugContext,
 		"xsltParseStylesheetProcess : found stylesheet\n");
 #endif
+	ret->literal_result = 0;
+	xsltParseStylesheetExcludePrefix(ret, cur, 1);
+	xsltParseStylesheetExtPrefix(ret, cur, 1);
+    } else {
+	xsltParseStylesheetExcludePrefix(ret, cur, 0);
+	xsltParseStylesheetExtPrefix(ret, cur, 0);
+	ret->literal_result = 1;
+    }
+    if (!ret->nopreproc) {
+#ifdef XSLT_REFACTORED
+	/*
+	* Reset the compiler context.
+	* TODO: This all looks ugly; but note that this will go
+	*   into an other function; it's just temporarily here.
+	*/
+	XSLT_CCTXT(ret)->depth = -1;
+	XSLT_CCTXT(ret)->inode = NULL;
+#endif
+	xsltPrecomputeStylesheet(ret, cur);
+#ifdef XSLT_REFACTORED
+	XSLT_CCTXT(ret)->depth = -1;
+	XSLT_CCTXT(ret)->inode = NULL;
+#endif
+    }
 
+    if (ret->literal_result == 0) {
 	xsltParseStylesheetTop(ret, cur);
     } else {
 	xmlChar *prop;
@@ -2064,7 +2445,7 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	/*
 	 * the document itself might be the template, check xsl:version
 	 */
-	prop = xsltGetNsProp(cur, (const xmlChar *)"version", XSLT_NAMESPACE);
+	prop = xmlGetNsProp(cur, (const xmlChar *)"version", XSLT_NAMESPACE);
 	if (prop == NULL) {
 	    xsltTransformError(NULL, ret, cur,
 		"xsltParseStylesheetProcess : document is not a stylesheet\n");
@@ -2102,6 +2483,7 @@ xsltParseStylesheetProcess(xsltStylesheetPtr ret, xmlDocPtr doc) {
 	template->elem = (xmlNodePtr) doc;
 	template->content = doc->children;
 	xsltAddTemplate(ret, template, NULL, NULL);
+	ret->literal_result = 1;
     }
 
     return(ret);
@@ -2128,6 +2510,7 @@ xsltParseStylesheetImportedDoc(xmlDocPtr doc, xsltStylesheetPtr style) {
     ret = xsltNewStylesheet();
     if (ret == NULL)
 	return(NULL);
+    
     if (doc->dict != NULL) {
         xmlDictFree(ret->dict);
 	ret->dict = doc->dict;
@@ -2176,7 +2559,17 @@ xsltParseStylesheetDoc(xmlDocPtr doc) {
 	return(NULL);
 
     xsltResolveStylesheetAttributeSet(ret);
-
+#ifdef XSLT_REFACTORED
+    /*
+    * Free the compilation context.
+    * TODO: Check if it's better to move this cleanup to
+    *   xsltParseStylesheetImportedDoc().
+    */
+    if (ret->compCtxt != NULL) {
+	xsltCompilerCtxtFree(XSLT_CCTXT(ret));
+	ret->compCtxt = NULL;
+    }
+#endif
     return(ret);
 }
 
